HTTP server module with server types and configuration.
Structs
- Server
- Synchronous HTTP server for request/response conversion.
- Server
Config - Server configuration for the HTTP/1.1 server.
- Shutdown
Controller - Controller for coordinated graceful shutdown.
- Shutdown
Receiver - Receiver for shutdown notifications.
- TcpServer
- TCP server with asupersync integration.
Enums
- Graceful
Outcome - Outcome of a task run with graceful shutdown support.
- Serve
Error - Error returned when starting or running the server fails.
- Server
Error - HTTP server error.
Constants
- DEFAULT_
DRAIN_ TIMEOUT_ SECS - Default drain timeout in seconds (time to wait for in-flight requests on shutdown).
- DEFAULT_
KEEP_ ALIVE_ TIMEOUT_ SECS - Default keep-alive timeout in seconds (time to wait for next request).
- DEFAULT_
MAX_ CONNECTIONS - Default maximum connections (0 = unlimited).
- DEFAULT_
MAX_ REQUESTS_ PER_ CONNECTION - Default max requests per connection (0 = unlimited).
- DEFAULT_
READ_ BUFFER_ SIZE - Default read buffer size in bytes.
- DEFAULT_
REQUEST_ TIMEOUT_ SECS - Default request timeout in seconds.
Functions
- serve
- Convenience function to serve an App on the given address.
- serve_
with_ config - Convenience function to serve an App with custom configuration.