pub struct SchemaRegistry { /* private fields */ }Expand description
Schema registry for #/components/schemas.
This owns a schema map and provides register() helpers that return $refs.
Implementations
Sourceimpl SchemaRegistry
impl SchemaRegistry
Sourcepub fn new() -> SchemaRegistry
pub fn new() -> SchemaRegistry
Create an empty registry.
Sourcepub fn register(&mut self, name: impl Into<String>, schema: Schema) -> Schema
pub fn register(&mut self, name: impl Into<String>, schema: Schema) -> Schema
Register schema under name if it doesn’t already exist, and return a $ref.
This does not overwrite an existing entry, which enables stable deduplication.
Sourcepub fn into_schemas(self) -> HashMap<String, Schema>
pub fn into_schemas(self) -> HashMap<String, Schema>
Consume the registry and return the underlying schema map.