Module testing

Module testing 

Testing utilities module.

Structs

CookieJar
A simple cookie jar for maintaining cookies across requests.
RequestBuilder
Builder for constructing test requests with a fluent API.
TestClient
Test client for in-process HTTP testing.
TestResponse
Response from a test request with assertion helpers.